@@ -4,7 +4,7 @@
 
     Qore Programming Language
 
-    Copyright (C) 2003 - 2022 Qore Technologies, s.r.o.
+    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -71,13 +71,13 @@ public:
     DLLEXPORT QoreStandardException(QoreStringNode* err, QoreStringNode* desc, QoreValue arg);
 
     //! Default move constructor
-    DLLEXPORT QoreStandardException(QoreStandardException&&) = default;
+    DLLLOCAL QoreStandardException(QoreStandardException&&) = default;
 
     //! Destroys the object
     DLLEXPORT virtual ~QoreStandardException();
 
     //! Default assignment operator
-    DLLEXPORT QoreStandardException& operator=(QoreStandardException&&) = default;
+    DLLLOCAL QoreStandardException& operator=(QoreStandardException&&) = default;
 
     //! Raises the corresponding Qore exception in the ExceptionSink.
     /** @param xsink the exception sink
