@@ -4,7 +4,7 @@
 
     Qore Programming Language
 
-    Copyright (C) 2003 - 2022 Qore Technologies, s.r.o.
+    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -241,4 +241,9 @@ DLLEXPORT extern const TypedHashDecl* hashdeclFtpResponseInfo;
 */
 DLLEXPORT extern const TypedHashDecl* hashdeclSocketPollInfo;
 
+//! PipeInfo hashdecl
+/** @since %Qore 1.12
+*/
+DLLEXPORT extern const TypedHashDecl* hashdeclPipeInfo;
+
 #endif
