@@ -2,7 +2,7 @@
 /*
     Qore Programming Language
 
-    Copyright (C) 2003 - 2022 Qore Technologies, s.r.o
+    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -32,41 +32,33 @@
 
 // include the right assembly macro file for the current architecture
 
-#ifdef __i386
-#include <qore/macros-i386.h>
-#endif // #ifdef __i386
-
 #ifdef __x86_64
 #include <qore/macros-x86_64.h>
-#endif // #ifdef __x86_64
-
-#ifdef __sparc
+#elif defined(__i386)
+#include <qore/macros-i386.h>
+#elif defined(__sparc)
 #include <qore/macros-sparc.h>
-#endif // #ifdef __sparc
-
-#if defined(__ppc) || defined(__ppc__)
-#include <qore/macros-powerpc.h>
-#endif // #ifdef __ppc
-
-#if defined(__ppc64) || defined(__ppc64__)
+#elif defined(__ppc64) || defined(__ppc64__) || defined(__powerpc64__) || defined(_ARCH_PPC64) || defined(__PPC64__)
 #include <qore/macros-ppc64.h>
-#endif // #ifdef __ppc64
-
-#ifdef __hppa
+#elif defined(__ppc) || defined(__ppc__) || defined(__powerpc__) || defined(_ARCH_PPC) || defined(__PPC__)
+#include <qore/macros-powerpc.h>
+#elif defined(__hppa)
 #include <qore/macros-parisc.h>
-#endif // #ifdef __hppa
-
-#ifdef __ia64
+#elif defined(__ia64)
 #include <qore/macros-ia64.h>
-#endif // #ifdef __ia64
-
-#ifdef __arm__
-#include <qore/macros-arm.h>
-#endif // #ifdef __arm__
-
-#ifdef __aarch64__
+#elif defined(__aarch64__)
 #include <qore/macros-aarch64.h>
-#endif // #ifdef __aarch64__
+#elif defined(__arm__)
+#include <qore/macros-arm.h>
+#elif defined(__riscv)
+#include <qore/macros-riscv.h>
+#elif (defined(__s390__) || defined(__s390x__) || defined(__zarch__))
+#include <qore/macros-s390x.h>
+#endif
+
+#ifndef HAVE_CHECK_STACK_POS
+#error need a solution to get the stack pointer with this compiler and on this CPU architecture
+#endif
 
 #endif // #ifndef _QORE_MACROS_H
 
